sdk_version = "0.0.11"
api_base = "https://v2.api.audio"
api_key = None

api_version = "0.0.11"
verify_ssl_certs = True
proxy = None
default_http_client = None
app_info = None
enable_telemetry = True
max_network_retries = 0


from audiostack import content as Content
from audiostack import speech as Speech
from audiostack import production as Production
from audiostack import delivery as Delivery
from audiostack.docs.docs import Documentation

billing_session = 0


def credits_used_in_this_session():
    return float("{:.2f}".format(billing_session))



from audiostack.helpers.request_interface import RequestInterface
from audiostack.helpers.request_types import RequestTypes
from audiostack.helpers.api_item import APIResponseItem
from audiostack.helpers.api_list import APIResponseList


class TTS:
    interface = RequestInterface(family="speech")
    
    class Item(APIResponseItem):
        def __init__(self, response) -> None:
            super().__init__(response)
            self.speechId = self.data["speechId"]

        def download(self, autoName=False, fileName="", path="./") -> None:
            sections = self.data["sections"]
            for i, s in enumerate(sections):
                if autoName:
                    full_name = ""
                    for k, val in s["audience"].items():
                        full_name += f"{k}={val}~"

                    full_name = full_name[:-1] + ".wav"
                else:
                    if not fileName:
                        full_name = s["sectionName"] + ".wav"
                    else:
                        full_name = f"{fileName}_{i+1}_of_{len(sections)}.wav"
                RequestInterface.download_url(
                    s["url"], destination=path, name=full_name
                )

        def delete(self):
            return TTS.delete(self.speechId)

    class BytesItem(APIResponseItem):
        def __init__(self, response) -> None:
            super().__init__(response)
            self.bytes = response["bytes"]
        # def download(self, autoName=False, fileName="default", path="./") -> None:
        #     with open("")



    class List(APIResponseList):
        def __init__(self, response, list_type) -> None:
            super().__init__(response, list_type)

        def resolve_item(self, list_type, item):
            if list_type == "speechIds":
                return TTS.Item({"data": item})
            else:
                raise Exception()

    class Section:
        @staticmethod    
        def create(
            sectionToProduce,
            scriptId="",
            scriptItem=None,
            voice: str = "",
            speed: float = 1.0,
            silencePadding: str = "",
            audience: dict = {},
            sections: dict = {},
            voiceIntelligence: bool = False,
            public: bool = False,
            sync: bool = True,
        ):
            # (start) no modify
            route = "tts/section"
            return TTS._create(**locals())
            # (end) modify

    @staticmethod
    def preview(text: str, voice: str):
        body = {
            "text" : text,
            "voice" : voice
        }
        r = TTS.interface.send_request(
            rtype=RequestTypes.POST, route="tts/preview", json=body
        )
        return TTS.BytesItem(r)

    @staticmethod
    def create(
        scriptId="",
        scriptItem=None,
        voice: str = "",
        speed: float = 1.0,
        silencePadding: str = "",
        audience: dict = {},
        sections: dict = {},
        voiceIntelligence: bool = False,
        public: bool = False,
        sync: bool = True,
    ) -> Item:
        # (start) no modify
        route = "tts"
        return TTS._create(**locals())
        # (end) modify
        


    @staticmethod
    def get(speechId: str) -> Item:
        r = TTS.interface.send_request(
            rtype=RequestTypes.GET, route="tts", path_parameters=speechId
        )
        return TTS.Item(r)

    @staticmethod
    def delete(speechId: str) -> str:
        r = TTS.interface.send_request(
            rtype=RequestTypes.DELETE, route="tts", path_parameters=speechId
        )
        return APIResponseItem(r)

    @staticmethod
    def list(
        projectName="", moduleName: str = "", scriptName: str = "", scriptId: str = ""
    ) -> list:
        query_params = {
            "projectName": projectName,
            "moduleName": moduleName,
            "scriptName": scriptName,
            "scriptId": scriptId,
        }
        r = TTS.interface.send_request(
            rtype=RequestTypes.GET, route="tts", query_parameters=query_params
        )
        return TTS.List(r, list_type="speechIds")


    @staticmethod
    def _create(
        route: str,
        scriptId="",
        scriptItem=None,
        voice: str = "",
        speed: float = 1.0,
        silencePadding: str = "",
        audience: dict = {},
        sections: dict = {},
        voiceIntelligence: bool = False,
        public: bool = False,
        sync: bool = True,
        sectionToProduce: str = ""
    ):
        if scriptId and scriptItem:
            raise Exception("scriptId or scriptItem should be supplied not both")
        if not (scriptId or scriptItem):
            raise Exception("scriptId or scriptItem should be supplied")

        if scriptItem:
            scriptId = scriptItem.scriptId

        if not isinstance(voice, str):
            raise Exception("voice argument should be a string")
        if not isinstance(silencePadding, str):
            raise Exception("silencePadding argument should be a string")


        body = {
            "scriptId": scriptId,
            "voice": voice,
            "speed": speed,
            "silencePadding": silencePadding,
            "audience": audience,
            "sections": sections,
            "voiceIntelligence": voiceIntelligence,
            "public": public,
            "sync": sync,
        }
        if sectionToProduce:
            body["sectionToProduce"] = sectionToProduce

        r = TTS.interface.send_request(rtype=RequestTypes.POST, route="tts", json=body)
        while r["statusCode"] == 202:
            print("Response in progress please wait...")
            r = TTS.interface.send_request(
                rtype=RequestTypes.GET, route=route, path_parameters=r["data"]["speechId"]   
            )
            
        return TTS.Item(r)


import json


class APIResponseItem:
    def __init__(self, response):
        self.response = response
        self.status_code = response["statusCode"]
        
        if "data" in response:
            self.data = self.response["data"]
        if "message" in response:
            self.message = self.response["message"]
        if "meta" in response:
            self.meta = self.response["meta"]
        if "bytes" in response:
            self.bytes = self.response["bytes"]

    def print_response(self, indent=0):
        if indent:
            return json.dumps(self.response, indent=indent)
        else:
            return self.response

    def __str__(self) -> str:
        if hasattr(self, "bytes"):
            return "bytes object"
        else:
            return json.dumps(self.response)



import json
from .api_item import APIResponseItem


class APIResponseList(APIResponseItem):
    def __init__(self, response, list_type):
        super().__init__(response)
        self.idx = 0
        self.list_type = list_type
        self.items = self.response["data"][list_type]

    def __iter__(self):
        return self

    def __next__(self):
        self.idx += 1
        try:
            item = self.items[self.idx - 1]
            return self.resolve_item(self.list_type, item)

        except IndexError:
            self.idx = 0
            raise StopIteration  

    def __getitem__(self, x):
        if isinstance(x, slice):
            return [self.resolve_item(self.list_type, x) for x in self.items[x.start: x.stop]]
        else:
            return self.resolve_item(self.list_type, x)
        

    # child classes should override this method, failing to do so will raise an exception!
    def resolve_item(self, list_type):
        raise Exception()



from audiostack.helpers.request_interface import RequestInterface
from audiostack.helpers.request_types import RequestTypes
from audiostack.helpers.api_item import APIResponseItem
from audiostack.helpers.api_list import APIResponseList


class Voice:
    interface = RequestInterface(family="speech/voice")

    class Item(APIResponseItem):
        def __init__(self, response) -> None:
            super().__init__(response)
            self.provider = self.data["provider"]
            self.alias = self.data["alias"]

    class List(APIResponseList):
        def __init__(self, response, list_type) -> None:
            super().__init__(response, list_type)

        def resolve_item(self, list_type, item):
            if list_type == "voices":
                return Voice.Item({"data": item})
            else:
                raise Exception()

    @staticmethod
    def list() -> list:
        r = Voice.interface.send_request(rtype=RequestTypes.GET, route="")
        return Voice.List(r, list_type="voices")

    class Parameter:
        @staticmethod
        def get() -> dict:
            r = Voice.interface.send_request(rtype=RequestTypes.GET, route="parameter")
            return APIResponseItem(r)
